#include "modest_config.h"

// myencoding
#include "third_party/modest/source/myencoding/encoding.c"
#include "third_party/modest/source/myencoding/detect.c"
#include "third_party/modest/source/myencoding/mystring.c"

// myunicode
#include "third_party/modest/source/myunicode/myosi.c"
