case MyHTML_TAG_ARTICLE:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_ASIDE:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_DETAILS:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_DIV:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_DT:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_FIGCAPTION:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_FOOTER:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_FORM:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_HEADER:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_HGROUP:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_HTML:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_MAIN:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_NAV:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_SECTION:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_SUMMARY:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_BODY:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_P:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_DL:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_DD:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_BLOCKQUOTE:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_FIGURE:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_ADDRESS:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_CENTER:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_H1:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_H2:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_H3:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_H4:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_H5:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_H6:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_XMP:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_PRE:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_PLAINTEXT:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_TABLE:
	return TAG_UA_STYLE_TABLE;
case MyHTML_TAG_CAPTION:
	return TAG_UA_STYLE_TABLE_CAPTION;
case MyHTML_TAG_TR:
	return TAG_UA_STYLE_TABLE_ROW;
case MyHTML_TAG_COL:
	return TAG_UA_STYLE_TABLE_COLUMN;
case MyHTML_TAG_COLGROUP:
	return TAG_UA_STYLE_TABLE_COLUMN_GROUP;
case MyHTML_TAG_TBODY:
	return TAG_UA_STYLE_TABLE_ROW_GROUP;
case MyHTML_TAG_THEAD:
	return TAG_UA_STYLE_TABLE_HEADER_GROUP;
case MyHTML_TAG_TFOOT:
	return TAG_UA_STYLE_TABLE_FOOTER_GROUP;
case MyHTML_TAG_TD:
	return TAG_UA_STYLE_TABLE_CELL;
case MyHTML_TAG_TH:
	return TAG_UA_STYLE_TABLE_CELL;
case MyHTML_TAG_UL:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_MENU:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_DIR:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_OL:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_LI:
	return TAG_UA_STYLE_LIST_ITEM;
case MyHTML_TAG_HR:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_FRAMESET:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_LINK:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_NOFRAMES:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_BASE:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_BASEFONT:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_DATALIST:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_HEAD:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_META:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_SCRIPT:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_STYLE:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_TITLE:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_NOEMBED:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_PARAM:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_TEMPLATE:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_AREA:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_MARQUEE:
	return TAG_UA_STYLE_INLINE_BLOCK;
case MyHTML_TAG_RUBY:
	return TAG_UA_STYLE_RUBY;
case MyHTML_TAG_RB:
	return TAG_UA_STYLE_RUBY_BASE;
case MyHTML_TAG_RP:
	return TAG_UA_STYLE_NONE;
case MyHTML_TAG_RT:
	return TAG_UA_STYLE_RUBY_TEXT;
case MyHTML_TAG_RTC:
	return TAG_UA_STYLE_RUBY_TEXT_CONTAINER;
case MyHTML_TAG_LEGEND:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_FIELDSET:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_SELECT:
	return TAG_UA_STYLE_INLINE_BLOCK;
case MyHTML_TAG_OPTION:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_OPTGROUP:
	return TAG_UA_STYLE_BLOCK;
case MyHTML_TAG_BUTTON:
	return TAG_UA_STYLE_INLINE_BLOCK;
case MyHTML_TAG_PROGRESS:
	return TAG_UA_STYLE_INLINE_BLOCK;
case MyHTML_TAG_METER:
	return TAG_UA_STYLE_INLINE_BLOCK;
